<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Listener;

use sly_Form;
use sly_Model_Medium;
use wv\metainfo\Authorisation;
use wv\metainfo\Metainfo;

class Medium extends Backend {
	/**
	 * Handler for SLY_MEDIA_ADDED
	 *
	 * adds all available metainfos to a new medium
	 *
	 * @param sly_Model_Medium $medium  a medium instance
	 */
	public function onMediumAdded(sly_Model_Medium $medium) {
		$container = Metainfo::getContainer();
		$metainfos = Metainfo::getAllMediumMetaInfos();
		$clangs    = \sly_Util_Language::findAll(true);
		$service   = $container['metainfo-service-value'];

		foreach ($metainfos as $name => $info) {
			$metainfos[$name] = $info->getDefault();
		}

		foreach ($clangs as $clang) {
			$service->insertMediumData($medium, $clang, $metainfos);
		}
	}

	/**
	 * Handler for SLY_MEDIA_FORM_EDIT
	 *
	 * Erzeugt das Frontend-Formular für Medien.
	 *
	 * @param  sly_Form $form    the form
	 * @param  array    $params  additional parameters
	 * @return sly_Form          the form
	 */
	public function onMediumEditForm(sly_Form $form, array $params) {
		if (!Authorisation::canEdit() && !Authorisation::canView()) {
			return $form;
		}

		$canEdit        = Authorisation::canEdit();
		$medium         = $params['medium'];
		$availableInfos = Metainfo::getAllMediumMetaInfos();
		$assignedData   = Metainfo::mediumData($medium, null, null, \WV_Sally::CLANG_ALL, true);
		$dataToDisplay  = $this->getInfosToDisplay($availableInfos, $assignedData, $availableInfos);

		if (!empty($availableInfos)) {
			$form->addHiddenValue('wv2metainfo', 1);
			require _WV2_PATH.'templates/media/meta_frontend.phtml';
		}

		return $form;
	}

	/**
	 * Handler für SLY_MEDIA_UPDATED
	 *
	 * Serialisiert das Frontend-Formular und gibt ihm Fehlerfalle die
	 * Fehlermeldung, sonst das Subject zurück.
	 *
	 * @param sly_Model_Medium $medium  a medium instance
	 */
	public function onMediumUpdated(sly_Model_Medium $medium) {
		if (!Authorisation::canEdit() || !sly_post('wv2metainfo', 'bool', false)) {
			return;
		}

		$container  = Metainfo::getContainer();
		$serializer = $container['metainfo-serializer'];
		$errors     = $serializer->processMediumForm($medium);

		if (is_array($errors)) {
			$this->showErrors($errors, Metainfo::TYPE_MEDIUM);
		}
	}

	/**
	 * Handler for SLY_MEDIA_DELETED
	 *
	 * Deletes the cache
	 *
	 * @param sly_Model_Medium $medium  a medium instance
	 */
	public function onMediumDeleted(sly_Model_Medium $medium) {
		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_MEDIA_RESTORED
	 *
	 * Deletes the cache
	 *
	 * @param sly_Model_Medium $medium  a medium instance
	 */
	public function onMediumRestored(sly_Model_Medium $medium) {
		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_MEDIA_PERMANENT_DELETED
	 *
	 * Deletes all metainfo values for a medium
	 *
	 * @param sly_Model_Medium $medium  a medium instance
	 */
	public function onMediumPermanentDeleted(sly_Model_Medium $medium) {
		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-value'];

		$service->deleteMedium($medium);

		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_MEDIA_USAGES
	 *
	 * Checks if a file is still in use.
	 *
	 * @param array $usages  usages found before
	 * @param array $params  additional parameters
	 */
	public function onMediaUsages(array $usages, array $params) {
		$container = Metainfo::getContainer();
		$sql       = $container->getPersistence();

		$sql->query(
			'SELECT object_id, clang, revision, meta_type FROM '.$sql->getPrefix().'wv2_meta '.
			'WHERE `value` LIKE '.$sql->quote('%'.$params['filename'].'%').' GROUP BY meta_type, object_id, clang'
		);

		foreach ($sql as $row) {
			$usage = array('object' => Metainfo::getObject($row['meta_type'], $row['object_id'], $row['clang'], $row['revision']));

			switch ($row['meta_type']) {
				case Metainfo::TYPE_ARTICLE:
					$usage['type'] = 'sly-article';
					break;

				case Metainfo::TYPE_CATEGORY:
					$usage['type'] = 'sly-category';
					break;

				case Metainfo::TYPE_MEDIUM:
					$usage['type'] = 'sly-medium';
					break;

				case Metainfo::TYPE_USER:
					$usage['type'] = 'sly-user';
					break;
			}

			$usages[] = $usage;
		}

		return $usages;
	}
}
