<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

class sly_Controller_Rbac extends sly_Controller_Backend implements sly_Controller_Interface {
	public function setRequest(sly_Request $request) {
		// transform JSON payload into regular form submission
		$json = $request->post('json', 'string');
		$data = !empty($json) ? json_decode($json, true) : null;

		if ($data && is_array($data)) {
			foreach ($data as $key => $value) {
				$request->post->set($key, $value);
			}
		}

		parent::setRequest($request);
	}

	private function pageHeader() {
		$container = $this->getContainer();
		$layout    = $container->getLayout();
		$devutils  = 'webvariants/developer-utils';

		if (sly_Util_AddOn::isAvailable($devutils)) {
			$layout->addJavaScriptFile(Util::addOnUri($devutils, 'js/jquery.cookie.min.js'));
			$layout->addJavaScriptFile(Util::addOnUri($devutils, 'js/jquery.wvtogglefieldset.min.js'));
			$layout->addJavaScriptFile(Util::addOnUri('webvariants/rbac', 'js/toggle.js'));
			$layout->addJavaScriptFile(Util::addOnUri('webvariants/rbac', 'js/jsonform.js'));
		}

		$layout->addCSSFile(Util::addOnUri('webvariants/rbac', 'css/backend.less'));
		$layout->pageHeader(t('rbac'));
	}

	public function indexAction() {
		$this->pageHeader();

		$hasRoles  = $this->listView();
		$config    = $this->getContainer()->getConfig();
		$showUsers = !$config->get('rbac/disable_user_list', false);

		if ($hasRoles && $showUsers) {
			$this->userlistView();
		}
	}

	protected function listView() {
		$service = pac_RoleService::getInstance();
		$roles   = $this->getRoles();
		$data    = array('roles' => array());

		foreach ($roles as $id => $role) {
			$count = $service->getUsersWithRole($role);
			$data['roles'][$id] = array('role' => $role, 'count' => $count);
		}

		$this->render('index.phtml', $data, false);

		return !empty($data['roles']);
	}

	protected function userlistView() {
		$userService  = $this->getContainer()->getUserService();
		$service      = pac_RoleService::getInstance();
		$roles        = $this->getRoles();
		$curUser      = $this->getCurrentUser();
		$canEditUsers = $curUser->isAdmin() || ($curUser->hasPermission('pages', 'user') && $curUser->hasPermission('user', 'edit'));
		$data         = array('roles' => array(), 'canEdit' => $canEditUsers);

		foreach ($roles as $id => $role) {
			$userIDs = $service->getAllUsersWithRole($role);
			$users   = array();

			foreach ($userIDs as $id) {
				$user = $userService->findById($id);
				if (!$user) continue;

				$users[$id] = $user->getName() ? $user->getName() : $user->getLogin();
			}

			uasort($users, 'strnatcasecmp');

			$data['roles'][] = array('role' => $role, 'users' => $users);
		}

		$this->render('userlist.phtml', $data, false);
	}

	public function addroleAction() {
		$save = sly_post('save', 'boolean', false);

		if ($save) {
			$title    = sly_post('role_title', 'string', '');
			$includes = sly_postArray('role_includes', 'int', array());

			try {
				pac_RoleService::getInstance()->createRole($title, $includes);
				$this->pageHeader();
				print sly_Helper_Message::info(t('rbac_role_added'));
				$this->listView();
				$this->userlistView();
			}
			catch (pac_Exception $e) {
				$this->pageHeader();
				print sly_Helper_Message::warn(t('rbac_error_while_adding_role', $e->getMessage()));

				$data = array(
					'role'          => null,
					'roleId'        => -1,
					'role_title'    => $title,
					'role_includes' => $includes,
					'title'         => t('rbac_add_role')
				);

				$this->render('edit_role.phtml', $data, false);
			}
		}
		else {
			$this->pageHeader();

			$data = array(
				'role'          => null,
				'roleId'        => -1,
				'role_title'    => '',
				'role_includes' => array(),
				'title'         => t('rbac_add_role')
			);

			$this->render('edit_role.phtml', $data, false);
		}
	}

	public function editroleAction() {
		$this->pageHeader();

		$roleService = pac_RoleService::getInstance();
		$roleId      = sly_request('role_id', 'int', -1);
		$save        = sly_post('save', 'boolean', false);
		$dispatcher  = sly_Core::dispatcher();

		$roleService->validate($roleId);

		if ($save) {
			$title    = sly_post('role_title', 'string', '');
			$includes = sly_postArray('role_includes', 'int', array());

			try {
				$roleService->editRole($roleId, $title, $includes);

				$permissions       = sly_post('wv42_perm', 'array');
				$destinationConfig = sly_Authorisation::getConfig();

				foreach ($destinationConfig as $destination => $config) {
					$destPermissions = isset($permissions[$destination]) ? $permissions[$destination] : array();
					$values          = array();

					if (isset($config['list'])) {
						foreach ($destPermissions as $objectId => $tokens) {
							foreach ($tokens as $token) {
								$values[$token][] = $objectId;
							}
						}
					}
					else {
						foreach ($destPermissions as $token) {
							$values[$token] = true;
						}
					}

					$values = $dispatcher->filter('WV42_DESTINATION_VALUES', $values, array('destination' => $destination, 'config' => $config));
					$roleService->setDestinationValues($roleId, $destination, $values);
				}

				print sly_Helper_Message::info(t('rbac_role_edited'));
			}
			catch (pac_Exception $e) {
				print sly_Helper_Message::warn(t('rbac_error_while_editing_role', $e->getMessage()));
			}
		}

		WV42_UserCache::flush();

		if ($_SERVER['REQUEST_METHOD'] === 'GET' || sly_post('apply', 'string')) {
			$role = $roleService->getRoleById($roleId);
			$data = array(
				'role'          => $role,
				'roleId'        => $roleId,
				'role_title'    => $role->getTitle(),
				'role_includes' => $role->getIncludes(),
				'action'        => 'index.php?page=rbac&func=doEditRole',
				'title'         => t('rbac_edit_role'),
				'roles'         => $roleService->getAllRoles()
			);

			$this->render('edit_role.phtml', $data, false);
		}
		else {
			$this->listView();
			$this->userlistView();
		}
	}

	public function deleteroleAction() {
		$roleId = sly_request('role_id', 'int', -1);

		$roleService = pac_RoleService::getInstance();
		$roleService->validate($roleId);
		$roleService->deleteRole($roleId);

		$this->pageHeader();
		print sly_Helper_Message::info(t('rbac_role_deleted'));

		if ($this->listView()) {
			$this->userlistView();
		}
	}

	protected function getViewFolder() {
		return SLY_RBAC_PATH.'views'.DIRECTORY_SEPARATOR;
	}

	public function checkPermission($action) {
		$request = $this->getRequest();

		if (($request->isMethod('POST') && in_array($action, array('addrole', 'editrole'))) || $action === 'deleterole') {
			sly_Util_Csrf::checkToken();
		}

		$user = $this->getCurrentUser();

		return WV42_RBAC::canManageRoles($user);
	}

	protected function getModules() {
		$service = $this->getContainer()->getModuleService();
		return $service->getModules();
	}

	protected function getStructure() {
		$rootCats        = sly_Util_Category::getRootCategories();
		$this->structure = array('0' => t('all'));

		if ($rootCats) {
			foreach ($rootCats as $rootCat) {
				$this->walkTree($rootCat, 0, $this->structure);
			}
		}

		return $this->structure;
	}

	protected function getMediaStructure() {
		$rootCats          = sly_Util_MediaCategory::getRootCategories();
		$this->mediaStruct = array('0' => t('all'));

		if ($rootCats) {
			foreach ($rootCats as $rootCat) {
				$this->walkTree($rootCat, 0, $this->mediaStruct);
			}
		}

		return $this->mediaStruct;
	}

	protected function walkTree($category, $depth, &$target) {
		if (empty($category)) return;

		$target[$category->getId()] = str_repeat(' ', $depth*2).$category->getName();

		$children = $category->getChildren();

		if (is_array($children)) {
			foreach ($children as $child) {
				$this->walkTree($child, $depth + 1, $target);
			}
		}
	}

	protected function getRoles() {
		$service = pac_RoleService::getInstance();
		$roles   = $service->getAllRoles();

		uasort($roles, array($this, 'compareRoles'));

		return $roles;
	}

	public function compareRoles(pac_Role $a, pac_Role $b) {
		return strnatcasecmp($a->getTitle(), $b->getTitle());
	}
}
